/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.output;

import fr.amap.lidar.amapvox.commons.Voxel;
import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author pverley
 */
public class TxtVoxelOutput extends AbstractOutput {

    public TxtVoxelOutput(Voxelization voxelisation) {
        super(voxelisation);
    }

    @Override
    public File[] write() throws IOException {

        File outputFile = voxCfg.getOutputFile();
        String fileName = outputFile.getName();
        LOGGER.info("writing file " + outputFile.getAbsolutePath());
        
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile))) {

            // headers
            writer.write(voxParam.infos.headerToString() + "\n");
            String sep = " ";
            List<String> outputFields = new ArrayList();

            // only write enabled output variables
            for (Voxel.OutputVariable variable : Voxel.OutputVariable.values()) {
                if (voxParam.isOutputVariableEnabled(variable)) {
                    outputFields.add(variable.getShortName());
                }
            }

            StringBuilder header = new StringBuilder();
            for (String field : outputFields) {
                header.append(field);
                header.append(sep);
            }
            writer.write(header.toString().trim() + "\n");

            // voxels
            int ivox = 0;
            int nvox = voxParam.infos.getSplit().x * voxParam.infos.getSplit().y * voxParam.infos.getSplit().z;
            for (int i = 0; i < voxParam.infos.getSplit().x; i++) {
                for (int j = 0; j < voxParam.infos.getSplit().y; j++) {
                    for (int k = 0; k < voxParam.infos.getSplit().z; k++) {
                        // task cancelled
                        if (voxelisation.isCancelled()) {
                            return null;
                        }
                        // current voxel
                        progress("writing file " + fileName, ivox++, nvox);
                        Voxel voxel = voxelisation.getVoxel(i, j, k);

                        // write line
                        writer.write(voxel.variablesToString(outputFields.toArray(new String[outputFields.size()]), voxCfg.getDecimalFormat()) + "\n");
                    }
                }
            }
        }
        return new File[] {outputFile};
    }

}
