package fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.faces;

import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.Face;
import java.util.Vector;
import org.joml.Vector2f;
import org.joml.Vector3f;

public final class FaceList {
    
    private final Vector<Face> faces;
    private int[] indices;
    private float[] normals;
    private float[] texcoords;

    public FaceList() {
        this.faces = new Vector();
        this.indices = null;
        this.normals = null;
        this.texcoords = null;
    }
    
    public final void addFace(final Face face) {
        faces.add(face);
    }
    
    public final boolean isEmpty() {
        return faces.isEmpty();
    }
    
    public final int[] getIndices() {
        if (indices == null) {
            final Vector<Integer> vertexList = new Vector();

            for (Face face : faces) {
                for (int index : face.getIndices()) {
                    vertexList.add(index);
                }
            }

            indices = new int[vertexList.size()];

            for (int i = 0; i < indices.length; i++) {
                indices[i] = vertexList.get(i);
            }
        }
        
        return indices;
    }
    
    public final float[] getNormals() {
        if (normals == null) {
            normals = new float[faces.size() * 3];

            int index = 0;

            for (Face face : faces) {
                final Vector3f normal = face.getNormal();
                normals[index] = normal.x;
                index++;
                normals[index] = normal.y;
                index++;
                normals[index] = normal.z;
                index++;
            }
        }
        
        return normals;
    }
    
    public final float[] getTexcoords() {
        if (texcoords == null) {
            texcoords = new float[faces.size() * 2];

            int index = 0;

            for (Face face : faces) {
                final Vector2f texcoord = face.getTexcoord();
                texcoords[index] = texcoord.x;
                index++;
                texcoords[index] = texcoord.y;
                index++;
            }
        }
        
        return texcoords;
    }
    
}
