/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.loadables.programs;

import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLContext;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.DrawingType;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.Mesh;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.extended.CubePrimitive;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.shaders.BorderShader;
import org.joml.Matrix4f;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class BorderProgram extends Program {
    
    /**
     * Projection matrix uniform value.
     */
    private Matrix4f proj;
    
    /**
     * Model matrix uniform value.
     */
    private Matrix4f model;
    
    /**
     * Voxel space width uniform value.
     */
    private int width;
    
    /**
     * Voxel space height uniform value.
     */
    private int height;
    
    /**
     * Voxel space depth uniform value.
     */
    private int depth;
    
    /* OpenGL uniform locations area */
    private int projLoc;
    private int modelLoc;
    private int widthLoc;
    private int heightLoc;
    private int depthLoc;

    public BorderProgram() {
        super();
        
        this.shader = BorderShader.getINSTANCE();
        this.mesh = new Mesh(CubePrimitive.getINSTANCE(), DrawingType.LINES);
        
        this.proj = new Matrix4f().identity();
        this.model = new Matrix4f().identity();
        this.width = 0;
        this.height = 0;
        this.depth = 0;
    }
    
    public final void setProj(final Matrix4f proj) {
        this.proj = proj;
    }
    
    public final void setModel(final Matrix4f model) {
        this.model = model;
    }
    
    public final void setWidth(final int width) {
        this.width = width;
    }

    public final void setHeight(final int height) {
        this.height = height;
    }

    public final void setDepth(final int depth) {
        this.depth = depth;
    }
    
    @Override
    protected void onLoad() {
        super.onLoad();
        
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
                
        /* Get the uniform locations */
        projLoc = gl.glGetUniformLocation(shader.getProgram(), "proj");
        modelLoc = gl.glGetUniformLocation(shader.getProgram(), "model");
        widthLoc = gl.glGetUniformLocation(shader.getProgram(), "width");
        heightLoc = gl.glGetUniformLocation(shader.getProgram(), "height");
        depthLoc = gl.glGetUniformLocation(shader.getProgram(), "depth");
    }
    
    @Override
    protected void onBind() {
        super.onBind();
        
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        
        /* Send uniform */
        gl.glUniformMatrix4fv(projLoc, 1, false, proj.get(new float[16]), 0);
        gl.glUniformMatrix4fv(modelLoc, 1, false, model.get(new float[16]), 0);
        gl.glUniform1i(widthLoc, width);
        gl.glUniform1i(heightLoc, height);
        gl.glUniform1i(depthLoc, depth);
        
        /* Bind attributes */
        mesh.bindPositions(0);
        
        /* Draw the number of instance */
        mesh.setInstances(1);
        mesh.draw();
    }
    
}
