/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.loadables.programs;

import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLContext;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.DrawingType;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.Mesh;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.extended.CubePrimitive;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.shaders.SimpleShader;
import java.awt.Color;
import org.joml.Matrix4f;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class SimpleProgram extends Program {
        
    /* GLSL uniforms area */
    
    /**
     * Projection matrix uniform value.
     */
    private Matrix4f proj;
    
    /**
     * Model matrix uniform value.
     */
    private Matrix4f model;
    
    /**
     * Scale matrix uniform value.
     */
    private Matrix4f view;

    /**
     * Voxel color value.
     */
    private Color mainColor;
    
    /* OpenGL uniform locations area */
    private int projLoc;
    private int modelLoc;
    private int viewLoc;
    private int mainColorLoc;

    public SimpleProgram() {
        super();
        
        this.shader = SimpleShader.getINSTANCE();
        this.mesh = new Mesh(CubePrimitive.getINSTANCE(), DrawingType.QUADS);
        
        this.proj = new Matrix4f().identity();
        this.model = new Matrix4f().identity();
        this.view = new Matrix4f().identity();
        this.mainColor = Color.WHITE;
    }
        
    public final void setProj(final Matrix4f proj) {
        this.proj = proj;
    }
    
    public final void setModel(final Matrix4f model) {
        this.model = model;
    }
    
    public final void setView(final Matrix4f view) {
        this.view = view;
    }

    public final void setMainColor(final Color mainColor) {
        this.mainColor = mainColor;
    }

    @Override
    protected void onLoad() {
        super.onLoad();
        
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        
        /* Get the uniform locations */
        projLoc = gl.glGetUniformLocation(shader.getProgram(), "proj");
        modelLoc = gl.glGetUniformLocation(shader.getProgram(), "model");
        viewLoc = gl.glGetUniformLocation(shader.getProgram(), "view");
        mainColorLoc = gl.glGetUniformLocation(shader.getProgram(), "mainColor");
    }

    @Override
    protected void onBind() {
        super.onBind();
        
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        
        /* Send uniform */
        gl.glUniformMatrix4fv(projLoc, 1, false, proj.get(new float[16]), 0);
        gl.glUniformMatrix4fv(modelLoc, 1, false, model.get(new float[16]), 0);
        gl.glUniformMatrix4fv(viewLoc, 1, false, view.get(new float[16]), 0);
        gl.glUniform4f(mainColorLoc, mainColor.getRed(), mainColor.getGreen(), 
                mainColor.getBlue(), mainColor.getAlpha());

        /* Draw the mesh */
        mesh.bindPositions(0);
        mesh.draw();
    }

    @Override
    protected void onUnload() {
        super.onUnload();
    }
    
}
