package fr.amap.lidar.amapvox.visualization.controllers.loadables.textures;

import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLContext;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;
import java.io.File;
import java.io.IOException;
import java.nio.Buffer;
import java.nio.ByteBuffer;

public class TextureCubemap extends Texture {
    
    private final Buffer[] images;

    public TextureCubemap(final int width, final int height,
            final Buffer right, final Buffer left,
            final Buffer top, final Buffer bottom,
            final Buffer back, final Buffer front) {
        super(width, height);
        this.images = new ByteBuffer[6];
        this.images[0] = right;
        this.images[1] = left;
        this.images[2] = top;
        this.images[3] = bottom;
        this.images[4] = back;
        this.images[5] = front;
    }
    
    public TextureCubemap(final int width, final int height,
            final String rightPath, final String leftPath,
            final String topPath, final String bottomPath,
            final String backPath, final String frontPath) throws IOException {
        super(width, height);
        this.images = new ByteBuffer[6];
        
        final TextureData right = TextureIO.newTextureData(
                GLProfile.getGL2GL3(), 
                getClass().getResource(rightPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, rightPath.split("\\.")[1]);
        images[0] = right.getBuffer();
                
        final TextureData left = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(leftPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, leftPath.split("\\.")[1]);
        images[1] = left.getBuffer();
        
        final TextureData bottom = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(bottomPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, bottomPath.split("\\.")[1]);
        images[2] = bottom.getBuffer();
        
        final TextureData top = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(topPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, topPath.split("\\.")[1]);
        images[3] = top.getBuffer();
        
        
        final TextureData back = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(backPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, backPath.split("\\.")[1]);
        images[4] = back.getBuffer();
        
        final TextureData front = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(frontPath), 
                GL3bc.GL_RGB, GL3bc.GL_RGB,
                false, frontPath.split("\\.")[1]);
        images[5] = front.getBuffer();
    }
    
    @Override
    protected void onLoad() {
        super.onLoad();
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, tbo.get(0));
        
        final char data;  
        for(int i = 0; i < images.length; i++)
        {
            gl.glTexImage2D(GL3bc.GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 
                0, GL3bc.GL_RGB, width, height, 0, GL3bc.GL_RGB, 
                GL3bc.GL_UNSIGNED_BYTE, images[i]
            );
        }
        
        gl.glGenerateMipmap(GL3bc.GL_TEXTURE_CUBE_MAP);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_MAG_FILTER, GL3bc.GL_LINEAR);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_MIN_FILTER, GL3bc.GL_LINEAR_MIPMAP_LINEAR);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_S, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_T, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_R, GL3bc.GL_CLAMP_TO_EDGE); 
    }
    
    @Override
    protected void onBind() {
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, tbo.get(0));
    }

    @Override
    protected void onUnbind() {
        final GL3bc gl = GLContext.getCurrentGL().getGL3bc();
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, 0);
    }
    
}
