/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.swing.jprogress.JProgressFrame;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.VoxelSpaceFile;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.Space;
import java.awt.BorderLayout;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class VisualizationFrame extends javax.swing.JFrame {

    /** Creates new form MainFrame */
    public VisualizationFrame() {
        initComponents();
        initChildren();
        setLocationRelativeTo(getParent());
    }
    
    public final void openFile() {
        final JFileChooser chooser = new JFileChooser();
        chooser.setFileFilter(new FileNameExtensionFilter("Voxel space files (.vox)", "vox"));
        chooser.setAcceptAllFileFilterUsed(false);
        
        int returnValue = chooser.showOpenDialog(this);

        if (returnValue == JFileChooser.APPROVE_OPTION) {
            final JProgressFrame progressFrame = new JProgressFrame(this);
            setEnabled(false);
            progressFrame.setVisible(true);
            
            final File selectedFile = chooser.getSelectedFile();
            final VoxelSpaceFile file = new VoxelSpaceFile();
                
            try {
                final InputStream in = new FileInputStream(selectedFile);
                
                progressFrame.setProgression(1, 3, "Parsing file...");
                
                if (file.parse(in)) {
                    java.awt.EventQueue.invokeLater(() -> {
                        try {
                            progressFrame.setProgression(2, 3, "Generate 3D scene...");
                            getView3DInternalFrame().getPanelVisualization().setSpace(
                                    new Space(file));

                            progressFrame.setProgression(3, 3, "Set properties...");
                            getPropertiesInternalFrame().setPropertyNames(file.getNames());

                            progressFrame.dispose();
                        setEnabled(true);
                        } catch(Exception ex) {
                            JOptionPane.showMessageDialog(this, 
                                    "Something wrong happened : " + ex.getMessage(), 
                                    "Error", JOptionPane.ERROR_MESSAGE);
                        }
                    });
                }
            } catch (FileNotFoundException ex) {
                
            }
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        view3DInternalFrame.getPanelVisualization().getGLWindow().destroy();
    }

    /** This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        jButtonOpen = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("AMAPVox - Visualization");
        setIconImage(new javax.swing.ImageIcon(getClass().
            getResource("/icons/visualization.png")).getImage());
    setMinimumSize(new java.awt.Dimension(640, 480));
    setPreferredSize(new java.awt.Dimension(900, 600));

    jToolBar1.setFloatable(false);

    jButtonOpen.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/document-open.png"))); // NOI18N
    jButtonOpen.setText("Open");
    jButtonOpen.setToolTipText("Open another Voxel Space File");
    jButtonOpen.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
    jButtonOpen.setFocusable(false);
    jButtonOpen.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
    jButtonOpen.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    jButtonOpen.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            jButtonOpenActionPerformed(evt);
        }
    });
    jToolBar1.add(jButtonOpen);

    getContentPane().add(jToolBar1, java.awt.BorderLayout.PAGE_START);

    pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonOpenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonOpenActionPerformed
        openFile();
    }//GEN-LAST:event_jButtonOpenActionPerformed

    /**
     * This method is called after the components initialization to add their
     * children.
     */
    private void initChildren() {
        propertiesInternalFrame = new PropertiesInternalFrame(this);
        add(propertiesInternalFrame, BorderLayout.WEST);

        view3DInternalFrame = new View3DInternalFrame();
        add(view3DInternalFrame, BorderLayout.CENTER);
    }
    
    /**
     * Get the view 3D internal frame.
     * @return 
     */
    public final View3DInternalFrame getView3DInternalFrame() {
        return view3DInternalFrame;
    }

    /**
     * Get the current properties internal frame.
     * @return 
     */
    public final PropertiesInternalFrame getPropertiesInternalFrame() {
        return propertiesInternalFrame;
    }
    
    /**
     * Get the current 3D voxel space object.
     * @return 
     */
    public final Space getSpace() {
        return view3DInternalFrame.getPanelVisualization().getSpace();
    }
    
    // Children declaration
    private PropertiesInternalFrame propertiesInternalFrame;
    private View3DInternalFrame view3DInternalFrame;
    // End of children declaration

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonOpen;
    private javax.swing.JToolBar jToolBar1;
    // End of variables declaration//GEN-END:variables
}
